<?php

namespace Drupal\bibliocommons\Plugin\Field\FieldFormatter;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandler;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\bibliocommons\BibliocommonsService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'Book' formatter.
 *
 * @FieldFormatter(
 *   id = "bibliocommons_book",
 *   label = @Translation("Book formatter"),
 *   field_types = {
 *     "string"
 *   }
 * )
 */
class BookFormatter extends FormatterBase {
  /**
   * Define type for API call and template to use.
   *
   * @var string
   */
  const TYPE = 'titles';

  /**
   * Constructs a BibliocommonsService instance.
   *
   * @param \Drupal\bibliocommons\BibliocommonsService $booksService
   *   The books service to integrate with bilbiocommons.
   */
  protected BibliocommonsService $booksService;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a module handler instance.
   *
   * @param \Drupal\Core\Extension\ModuleHandler $moduleHandler
   *   Module handler.
   */
  protected ModuleHandler $moduleHandler;

  /**
   * Constructs a BookListFormatter instance.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings settings.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory service.
   * @param \Drupal\bibliocommons\BibliocommonsService $booksService
   *   The books service to integrate with bilbiocommons.
   * @param \Drupal\Core\Extension\ModuleHandler $moduleHandler
   *   The module handler to get module related information.
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    $label,
    $view_mode,
    array $third_party_settings,
    ConfigFactoryInterface $configFactory,
    BibliocommonsService $booksService,
    ModuleHandler $moduleHandler,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->configFactory = $configFactory;
    $this->booksService = $booksService;
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('config.factory'),
      $container->get('bibliocommons.books_service'),
      $container->get('module_handler'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $result = [];
    $fields = $items->getValue();

    foreach ($fields as $field) {
      // Continue if the entity doesn't have the required fields.
      $value = $field["value"];

      if ($value) {
        $result = $this->buildBook($value);
        break;
      }
    }

    return $result;
  }

  /**
   * Constructs a book array.
   *
   * @param int $book_id
   *   The book id to grab the book information.
   *
   * @return array
   *   The render array for books.
   */
  private function buildBook($book_id) {
    $image_base_url = 'https://secure.syndetics.com/index.aspx?isbn=';
    $min_image_width = 5;
    $module_path = $this->moduleHandler->getModule('bibliocommons')->getPath();
    $config = $this->configFactory->get('bibliocommons');

    // Set up the render array.
    $result = [
      '#theme' => 'bibliocommons_books',
      '#books' => [],
      '#title' => '',
      '#cover_url' => '',
      '#details_url' => '',
      '#type' => 'titles',
      '#attached' => [
        'library' => ['bibliocommons/bibliocommons'],
      ],
    ];

    // Get book by ID with request.
    $book_id = intval($book_id);
    $books = $this->booksService->getBookList($book_id, 1, self::TYPE);

    // Handle the book if we have one.
    if (count($books) > 0) {
      $book = $books['title'];

      // Add the book title and url.
      $result['#title'] = $book['title'];
      $result['#details_url'] = $book["details_url"];

      // Set the cover URL.
      $book_class = !empty($book['format']['id']) ? strtolower($book['format']['id']) : 'bk';
      $cover_url = '/' . $module_path . '/assets/images/icon-' . $book_class . '.png';

      $image_isbn = $book['isbns'][0] ?? NULL;

      if ($image_isbn) {
        $image_upc = $book['upcs'][0] ?? NULL;
        $remote_cover_url = $image_base_url . $image_isbn . '/MC.GIF&client=' . $config->get('client_id') . '&upc=' . $image_upc;
        $remote_cover_size = getimagesize($remote_cover_url);

        // When the image doesn't exist but we have a valid book we
        // get a single pixel. So check the size to determine if the image
        // is valid.
        if (!empty($remote_cover_size[0]) && $remote_cover_size[0] > $min_image_width) {
          $cover_url = $remote_cover_url;
        }
      }

      $result['#cover_url'] = $cover_url;
    }

    return $result;
  }

}
